library(tibble)
get_pv <- function(comps, data, col.x, col.y){
  
  group1 <- data[,col.y][data[ ,col.x] == comps[1]] # sample 1
  group2 <- data[,col.y][data[ ,col.x] == comps[2]] # sample 2
  alternative <- comps[3] # "two.sided", "less", "greater"
  
  print("head de group1")
  print(head(group1))
  print("head de group2")
  print(head(group2))
  
  if (is.na(alternative)) {
    med1 <- median(group1, na.rm=TRUE)
    med2 <- median(group2, na.rm=TRUE)
    print(paste("med1 :", med1))
    print(paste("med2 :", med2))
    if (med1 > med2){
      alternative <- "greater"
    } else if (med1 < med2){
      alternative <- "less"
    } else { alternative <- "two.sided"
    }
  } else if(! alternative %in% c("two.sided", "less", "greater")) {
    stop("'alternative' :", alternative," is not valid. Choose among 'two.sided', 'less', 'greater'.")
  }
  
  print(paste("Number of element in group", comps[1],":",length(group1)))
  print(paste("Number of element in group", comps[2],":",length(group2)))
  print(paste("Alternative used :", alternative))
  
  pvalues = NULL
  if (length(group1) > 500 || length(group2) > 500){
    for (i in 1:100){
      if (length(group1) > 500){
        s1 = sample(group1, size = 500)
      } else {
        s1 = group1
      }
      if (length(group2) > 500){
        s2 = sample(group2, size = 500)
      } else {
        s2 = group2
      }
      pval.tmp = wilcox.test(s1, s2, alternative = alternative, exact = FALSE)$p.value
      pvalues = c(pvalues,pval.tmp)
    }
  } else {
    pvalues = c(wilcox.test(group1, group2, alternative = alternative, exact = FALSE)$p.value)
  }
  
  print(paste("Number of pvalues :",length(pvalues)))
  print(head(pvalues))
  pv <- mean(pvalues, na.rm=TRUE)
  if (is.nan(pv)){
    pv <- 1
  }
  
  print(paste("mean pvalues :", pv))
  return(pv)
}
get_asterisks = function(x){
  if(x > 0.05){return("ns")}
  else if(x <= 0.05 & x > 0.01){return("*")}
  else if(x <= 0.01 & x > 0.001){return("**")}
  else if(x <= 0.001){return("***")}
}
stat_pvalue_manual_data <- function(data, comps, col.x, col.y){
  #' Return a tibble object that can be provided to :
  #' stat_pvalue_manual(data = tibble)
  #' to add significance on a ggplot object.
  #' Initially thought for boxplots, violinplots, etc...
  #' 
  #' data : dataframe containning col.x and col.y
  #' comps : list object with groups to compare (e.g. 'list( c("A","B","less"), c("B","C","greater")')
  #' col.x : name of the grouping column (factor) with comps among its levels
  #' col.y : name of the value column

  pvs <- lapply(X = comps, FUN = get_pv, data = data, col.x = col.x, col.y = col.y) # collect the p-values
  print(paste("p-values :", pvs))
  p.adj <- p.adjust(p = pvs, method = "bonferroni") # adjust the multiple p-values with the selected method
  p.adj.signif <- lapply(X = p.adj, FUN = get_asterisks) # translate them into asterisks.
  
  stat.test <- tibble(group1 = unlist(lapply(comps, `[[`, 1)),
                      group2 = unlist(lapply(comps, `[[`, 2)),
                      p = pvs,
                      p.adj = p.adj,
                      p.adj.signif = p.adj.signif)
  
  # As levels with less than 2 points will not be plotted, do not consider these in the levels order when adding the significance. on the plot.
  # real_order <- names(table(df$FILTER)[table(df$FILTER) >= 2])
  # stat.test$xmin <- match(stat.test$group1, real_order)
  # stat.test$xmax <- match(stat.test$group2, real_order)
  
  print(stat.test)
  return(stat.test)
}

# Example
# get_exclusive_plot(NAME = "SCER", feature = "size")+ 
#   stat_pvalue_manual(data = stat_pvalue_manual_data(data = melted_df, 
#                                                     comps = list(c("nonTRG", "hasNcHit", "greater")),
#                                                     col.x = "variable",
#                                                     col.y = "size"), y.position = 50)
